/********* Mail01 *******

  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp32-email-alert-temperature-threshold/
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files.
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
  
  Edit & Redesign: Nebojsa Urosevic
                   www.e-stan.eu 
		   info@edata.hr 
*********/

#include <WiFi.h>               // Enables network connection (local and Internet) using the Arduino WiFi shield.
#include <AsyncTCP.h>           // This is a fully asynchronous TCP library
#include <ESPAsyncWebServer.h>  // Async HTTP and WebSocket Server
#include <OneWire.h>            // Access 1-wire devices as temperature sensors
#include <DallasTemperature.h>  // Arduino Library for Dallas Temperature ICs
#include "ESP32_MailClient.h"   // Mail Client Arduino Library for ESP32
#include "time.h"               // Date and Time functions, with provisions to synchronize to external time sources like 
                                // GPS and NTP (Internet).

// REPLACE WITH YOUR NETWORK CREDENTIALS
const char* ssid = "------";
const char* password = "--------";

// To send Email using Gmail use port 465 (SSL) and SMTP Server smtp.gmail.com
const String emailSenderAccount  = "your mail for send";    // <<<<<<<
const String emailSenderPassword = "your mail password";    // <<<<<<<
const String smtpServer          = "smtp.gmail.com";        // IF Gmail account
const int    smtpServerPort      = 465;
      String emailSubject        = "[ALERT] KC868-A4 Temperature";
const int    Rel01 = 2;
const int    Rel02 = 15;
const int    Rel03 = 5;
const int    Rel04 = 4;
const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = 3600;
const int   daylightOffset_sec = 3600;

String inputMessage = "your mail for recipient";  // <<<<<<< Default Recipient Email Address
String enableEmailChecked = "checked";
String inputMessage2 = "true";
String inputMessage3 = "30.0";  // Default Threshold Temperature Value
String lastTemperature;
float temperature;
float previoustemp = 0.0;
unsigned long currentMillis;
bool stay = false;

// HTML web page to handle 3 input fields (email_input, enable_email_input, threshold_input)
// PROGMEM store data in flash (program) memory instead of SRAM.
const char index_html[] PROGMEM = R"rawliteral(

<!DOCTYPE HTML>
 <html>
   <head>

    <title>Email Notification with Temperature</title>
    <meta name = "viewport" content = "width = device-width, initial-scale = 1">
  </head>
  <body>
    <h1 style="background-color:DodgerBlue;"> Temperature monitoring</h1>
    <h1>%TEMPERATURE% &deg;C</h1>    
    <h3 style="background-color:Tomato;"> Kincony KC868-A4 Email Notification</h3>
    <p style="color:MediumSeaGreen;"> When the temperature exceeds the limit, a notification mail will be sent.</p>
    <p style="color:MediumSeaGreen;"> When the temperature returns below the limit, a notification mail will be sent..</p>
    
    <form action="/get">
      Email Address <input type = "email" name = "email_input" value = "%EMAIL_INPUT%" required> <br>
      Enable Email Notification <input type = "checkbox" name = "enable_email_input" value = "true" %ENABLE_EMAIL%> <br>
      Temperature Limit <input type = "number" step="0.1" name = "threshold_input" value = "%THRESHOLD%" required> <br>
    <input type = "submit" value = "Submit">
    </form>

  </body> 
 </html>
 )rawliteral";

void notFound(AsyncWebServerRequest *request) 
{
  request->send(404, "text/plain", "Not found");
}

AsyncWebServer server(80);

// Replaces placeholder with DS18B20 values
String processor(const String& var)
{
  //Serial.println(var);
  if(var == "TEMPERATURE")
  {
    return lastTemperature;
  }
  else if(var == "EMAIL_INPUT")
  {
    return inputMessage;
  }
  else if(var == "ENABLE_EMAIL")
  {
    return enableEmailChecked;
  }
  else if(var == "THRESHOLD")
  {
    return inputMessage3;
  }
  return String();
}

// Flag variable to keep track if email notification was sent or not
bool emailSent = false;

const char* PARAM_INPUT_1 = "email_input";
const char* PARAM_INPUT_2 = "enable_email_input";
const char* PARAM_INPUT_3 = "threshold_input";

unsigned long previousMillis = 0;     
const long interval = 5000;    

// GPIO where the DS18B20 is connected to
const int oneWireBus = 13;    //GPIO DS18B20 KC868-A4
// Setup a oneWire instance to communicate with any OneWire devices
OneWire oneWire(oneWireBus);
// Pass our oneWire reference to Dallas Temperature sensor 
DallasTemperature sensors(&oneWire);

// The Email Sending data object contains config and data to send
SMTPData smtpData;

void setup() 
{
  pinMode(Rel01, OUTPUT);
  pinMode(Rel02, OUTPUT);
  pinMode(Rel03, OUTPUT);
  pinMode(Rel04, OUTPUT);
  
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  // Connect to Wifi network:
  Serial.print("Connecting to Wifi SSID ");
  while (WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(500);
  }

  Serial.println();
  Serial.print("KC868-A4 IP Address: http://");
  Serial.println(WiFi.localIP());

  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
 // printLocalTime();


 //Serial.print(index_html);  // Test HTML contents
  
  // Start the DS18B20 sensor
  sensors.begin();

  // Send web page to client
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    request->send_P(200, "text/html", index_html, processor);  //const char index_html[] PROGMEM = R"rawliteral(
  }  );

  // Receive an HTTP GET request at <ESP_IP>/get?email_input=<inputMessage>&enable_email_input=<inputMessage2>&threshold_input=<inputMessage3>
  server.on("/get", HTTP_GET, [] (AsyncWebServerRequest *request) 
  {
    // GET email_input value on <ESP_IP>/get?email_input=<inputMessage>
    if (request->hasParam(PARAM_INPUT_1)) 
    {
      inputMessage = request->getParam(PARAM_INPUT_1)->value();
      // GET enable_email_input value on <ESP_IP>/get?enable_email_input=<inputMessage2>
      if (request->hasParam(PARAM_INPUT_2)) 
      {
        inputMessage2 = request->getParam(PARAM_INPUT_2)->value();
        enableEmailChecked = "checked";
      }
      else 
      {
        inputMessage2 = "false";
        enableEmailChecked = "";
      }
      // GET threshold_input value on <ESP_IP>/get?threshold_input=<inputMessage3>
      if (request->hasParam(PARAM_INPUT_3)) 
      {
        inputMessage3 = request->getParam(PARAM_INPUT_3)->value();
      }
    }
    else 
    {
      inputMessage = "No message sent";
    }
    Serial.println(inputMessage);
    Serial.println(inputMessage2);
    Serial.println(inputMessage3);
    request->send(200, "text/html", "HTTP GET request sent to your ESP.<br><a href=\"/\">Return to Home Page</a>");
  }  );
  server.onNotFound(notFound);
  server.begin();
}

void loop() 
{
  currentMillis = millis();
  if (currentMillis - previousMillis >= interval) 
  {
    previousMillis = currentMillis;
    sensors.requestTemperatures();
    // Temperature in Celsius degrees 
   
    temperature = sensors.getTempCByIndex(0);
      
    if (previoustemp != temperature)
    {
      TempTest();       
    }

    // Temperature in Fahrenheit degrees
    /*float temperature = sensors.getTempFByIndex(0);
    SerialMon.print(temperature);
    SerialMon.println(" °F");*/
    
    lastTemperature = String(temperature);
    
    // Check if temperature is above threshold and if it needs to send the Email alert
    if(temperature > inputMessage3.toFloat() && inputMessage2 == "true" && !emailSent)
    {
      String emailMessage = String("Temperature above the limit. \nCurrent temperature: ") + 
                            String(temperature) + String("°C");
      if(sendEmailNotification(emailMessage)) 
      {
        Serial.println("-- Mail sent --");
        Serial.println(emailMessage);
        Serial.println("----------------------");
        emailSent = true;
      }
      else 
      {
        Serial.println("Email failed to send");
      }    
    }
    // Check if temperature is below threshold and if it needs to send the Email alert
    else if((temperature < inputMessage3.toFloat()) && inputMessage2 == "true" && emailSent) 
    {
      String emailMessage = String("The temperature returned below the limit. Current temperature: ") + 
                            String(temperature) + String("°C");
      if(sendEmailNotification(emailMessage)) 
      {
        Serial.println("-- Mail sent --");
        Serial.println(emailMessage);
        Serial.println("----------------------");
        
        emailSent = false;
      }
      else 
      {
        Serial.println("Email failed to send");
      }
    }
  }
}

bool sendEmailNotification(String emailMessage)

{
  // Set the SMTP Server Email host, port, account and password
  smtpData.setLogin(smtpServer, smtpServerPort, emailSenderAccount, emailSenderPassword);

  // For library version 1.2.0 and later which STARTTLS protocol was supported,the STARTTLS will be 
  // enabled automatically when port 587 was used, or enable it manually using setSTARTTLS function.
  // smtpData.setSTARTTLS(true);

  // Set the sender name and Email
  smtpData.setSender("Temp-Alert", emailSenderAccount);

  // Set Email priority or importance High, Normal, Low or 1 to 5 (1 is highest)
  smtpData.setPriority("High");

  // Set the subject
  smtpData.setSubject(emailSubject);

  // Set the message with HTML format
  smtpData.setMessage(emailMessage, true);

  // Add recipients
  smtpData.addRecipient(inputMessage);

  smtpData.setSendCallback(sendCallback);

  // Start sending Email, can be set callback function to track the status
  if (!MailClient.sendMail(smtpData)) 
  {
    Serial.println("Error sending Email, " + MailClient.smtpErrorReason());
    return false;
  }
  // Clear all data from Email object to free memory
  smtpData.empty();
  return true;
}

void sendCallback(SendStatus msg)  // Callback function to get the Email sending status
{
 
  // Serial.println(msg.info());   // Print the current status

  // Do something when complete
  if (msg.success()) 
  {
    Serial.println("----------------------");
  }
}

void TempTest()
{  
    previoustemp = temperature;
    printLocalTime();
    Serial.print("Temperature: ");
    Serial.print(temperature);
    Serial.println("°C  ");
  
 if(temperature > inputMessage3.toFloat() && temperature < (inputMessage3.toFloat()+4) && inputMessage2 == "true" && stay == false)
   {
    Serial.print("Fan -> ON;  ");
    Serial.println("Wall -> Close");
    Serial.println("--------------------------------");
    digitalWrite(Rel01, HIGH);   // Start cooling
    digitalWrite(Rel02, LOW);   // Opening wall side
    emailSubject        = "[ALERT] KC868-A4 Temperature **HIGH**";
   }
   else if(temperature > inputMessage3.toFloat() && temperature < (inputMessage3.toFloat()+4) && inputMessage2 == "true" && stay == true)
   {
    Serial.print("Fan -> ON;  ");
    Serial.println("Wall -> Open");
    Serial.println("--------------------------------");
    digitalWrite(Rel01, HIGH);   // Start cooling
    digitalWrite(Rel02, HIGH);   // Opening wall side
   }
   else if(temperature < inputMessage3.toFloat() && inputMessage2 == "true") 
   {
    stay = false;   // end delay wall closing
    Serial.print("Fan -> OFF;  ");
    Serial.println("Wall -> Close");
    Serial.println("--------------------------------");
    digitalWrite(Rel01, LOW);   // Stop cooling
    digitalWrite(Rel02, LOW);   // Closing wall side   
    emailSubject        = "[ALERT] KC868-A4 Temperature **NORMAL**";
   }          
   else if(temperature > (inputMessage3.toFloat()+4) && inputMessage2 == "true")
   {
    stay = true;    // delay wall closing
    Serial.print("Fan -> ON;  ");
    Serial.println("Wall -> Open");
    Serial.println("--------------------------------");
    digitalWrite(Rel01, HIGH);   // Stop cooling
    digitalWrite(Rel02, HIGH);   // Opening wall side
   }

}

void printLocalTime()
{
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.print(&timeinfo, "%A, %B %d %Y %H:%M:%S");
  Serial.print(" >> ");
}
